/* psprite.c
**
** pseudo-Sprites+ example
**
** Just one method to do sprites using KGL + cull backface
**
** Use the A-button to toggle transparency, the B-button to toggle
** backface culling, and the Y-button to toggle the background.
**
** bmp routines taken from DC Tonic Cool Herders Demo
**
** (c) Paul Boese a.k.a. Axlen, Nov 2001
**
** rev 0.0.0
*/

#include <kos.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "bmp.h"

/* Load a texture using pcx_load_texture and glKosTex2D */
void loadtxr(const char *fn, GLuint *txr, GLint texformat, int alpha) {
	uint32 txaddr;
	int w, h;
	if (pcx_load_texture(fn, 1, alpha, &txaddr, &w, &h) < 0) {
		printf("can't load %s\n", fn);
		return;
	}

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(texformat, w, h, txaddr);
}

// Loads a BMP file to a texture  - Use with transparent polys!
// BMP file should be at least 256x256.
// Black (R=G=B=0) is transparent.
void loadbmptxr(const char *fn, GLuint *txr) {
	uint32 txaddr;
	int w, h;

	BITMAP *myBmp;

	myBmp=load_bmp(fn);
	if (NULL != myBmp) {
		w = myBmp->nWidth;
		h = myBmp->nHeight;
		txaddr = ta_txr_allocate(w * h *2);
		ta_txr_load(txaddr, myBmp->data, w * h *2); 
		//txr_to_alpha(myBmp->data, w, h);
		//txr_twiddle_copy(myBmp->data, w, h, txaddr, w, h, 0);
		free(myBmp);
		printf("%s %dx%d\n", fn, w, h);
	} else {
		printf("*** %s didn't load ***\n", fn);
		return;
	}
	
	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_ARGB1555, w, h, txaddr);
}

void back_draw(GLuint texture) {

	glEnable(GL_KOS_AUTO_UV);
	glBindTexture( GL_TEXTURE_2D, texture);

	// Scale (read stretch) the image to full screen size.
	// Original image was 640x480, but had to be squished into a 512x512
	// square to make the pvr happy.  Here we essentially un-squish it.
	// This method seems to eat a lot of the pvr cycles.  Not good!
	glTranslatef(0.0f, 0.0f, -(480.0f/640.0f));
	glScalef(640.0f/480.0f, 1.0f, 0.0f);
	
	glBegin(GL_QUADS);
		
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);
	
	glEnd();
	glDisable(GL_KOS_AUTO_UV);
}

void sprite_draw(GLuint texture) {

	glEnable(GL_KOS_AUTO_UV);
	glBindTexture( GL_TEXTURE_2D, texture);

	glBegin(GL_QUADS);
		
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);

	glEnd();
	glDisable(GL_KOS_AUTO_UV);
}

void cube(float r, GLuint texture) {
	glEnable(GL_KOS_AUTO_UV);
	glBindTexture( GL_TEXTURE_2D, texture);

	glBegin(GL_QUADS);
	glRotatef(r, 1.0f, 0.0f, 1.0f);

	/* Front face */
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, 1.0f, 1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);

	/* Back face */
	glVertex3f(-1.0f, -1.0f, -1.0f);
	glVertex3f(1.0f, -1.0f, -1.0f);
	glVertex3f(1.0f, 1.0f, -1.0f);
	glVertex3f(-1.0f, 1.0f, -1.0f);
                        
	/* Left face */
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);
	glVertex3f(-1.0f, -1.0f, -1.0f);
	glVertex3f(-1.0f, 1.0f, -1.0f);

	/* Right face */
	glVertex3f(1.0f, 1.0f, -1.0f);
	glVertex3f(1.0f, -1.0f, -1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glVertex3f(1.0f, 1.0f, 1.0f);

	/* Top face */
	glVertex3f(1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, -1.0f);
	glVertex3f(1.0f, 1.0f, -1.0f);

	/* Bottom face */
	glVertex3f(1.0f, -1.0f, -1.0f);
	glVertex3f(-1.0f, -1.0f, -1.0f);
	glVertex3f(-1.0f, -1.0f, 1.0f);
	glVertex3f(1.0f, -1.0f, 1.0f);
	glEnd();
	glDisable(GL_KOS_AUTO_UV);
}

extern uint8 romdisk[];
GLuint texfont;

int main(void) {
	uint8 c;
	cont_cond_t cond;
	GLuint tex_axlen, tex_back, tex_roundel, tex_knot;
	float r, dr = F_PI*2/120.0f; 
	int cullface = 0;
	int drawtrans = 0;
	int back = 0;

	/* Initialize KOS */
	printf("[pseudo-Sprites+] Kos Initialing\n");

	kos_init_all(THD_ENABLE | TA_ENABLE | IRQ_ENABLE, romdisk);

	printf("[pseudo-Sprites+] glKos Initialing\n");

	/* Get basic stuff initialized */
	glKosInit();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	gluPerspective(30.0f, 640.0f / 480.0f, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_TEXTURE_2D);
	c = maple_first_controller();
	
	printf("[pseudo-Sprites+] Loading Textures\n");
	loadbmptxr("/rd/back.bmp", &tex_back);
	loadbmptxr("/rd/axlen.bmp", &tex_axlen);
	loadbmptxr("/rd/roundel.bmp", &tex_roundel);
	loadbmptxr("/rd/squareknot.bmp", &tex_knot);

	/* pseudo-Sprites+ Main loop */
	printf("[pseudo-Sprites+] Entering Main Loop\n");
	while(1) {
		/* Check key status */
		if (cont_get_cond(c, &cond) < 0)
			break;
		if (!(cond.buttons & CONT_START))
			break;

		/* toggle background */
		if (!(cond.buttons & CONT_Y)) {
			if (!(back & 0x1000)) {
				if (back == 0)
					back = 0x1001;
				else
					back = 0x1000;
			}
		} else
			back &= ~0x1000;

		/* toggle backface culling */
		if (!(cond.buttons & CONT_B)) {
			if (!(cullface & 0x1000)) {
				if (cullface == 0)
					cullface = 0x1001;
				else
					cullface = 0x1000;
			}
		} else
			cullface &= ~0x1000;

		/* toggle transparency */
		if (!(cond.buttons & CONT_A)) {
			if (!(drawtrans & 0x1000)) {
				if (drawtrans == 0)
					drawtrans = 0x1001;
				else
					drawtrans = 0x1000;
			}
		} else
			drawtrans &= ~0x1000;


		r += dr;

		/* Begin frame */
		glKosBeginFrame();
		glLoadIdentity();
		
		if (!back) {
			glPushMatrix();
			back_draw(tex_back);
			glPopMatrix();
		}

		/* Done with Opaque Polys */
		if (!drawtrans & 1) glKosFinishList();
		if (!cullface & 1) glDisable(GL_CULL_FACE);
		
		/* start of Transparent Polys */
		glPushMatrix();
		glTranslatef(fsin(r)/18.0f, fcos(r)/18.0f, -0.1f);
		glScalef(0.3f, 0.3f, 0.3f);
		glRotatef(r, 0.0f, 0.5f, 0.0f);
		sprite_draw(tex_axlen);
		glPopMatrix();
		
		/* Center cube */
		glPushMatrix();
		glScalef(0.2f, 0.2f, 0.2f);
		glTranslatef(0.0f, 0.0f, -1.0f);
		cube(r/4.0f, tex_roundel);
		glPopMatrix();

		/* Corner cubes Trans */
		glPushMatrix();
		glScalef(0.1f, 0.1f, 0.1f);
		glTranslatef(-4.9f, -2.9f, -1.0f);
		cube(r/4.0f, tex_knot);
		glPopMatrix();

		glPushMatrix();
		glScalef(0.1f, 0.1f, 0.1f);
		glTranslatef(4.9f, 2.9f, -1.0f);
		cube(r/4.0f, tex_knot);
		glPopMatrix();

		glEnable(GL_CULL_FACE);

		/* Finish the frame */
		glKosFinishFrame();
	}

	printf("[pseudo-Sprites+] Exited main loop\n");
	kos_shutdown_all();
	return 0;
}

/* vi:tabstop=4
*/	
